@@ -47,14 +47,37 @@ G_DEFINE_TYPE_WITH_PRIVATE (TrackerResource, tracker_resource, G_TYPE_OBJECT)
 #define GET_PRIVATE(object)  (tracker_resource_get_instance_private (object))
 
 /**
- * SECTION: tracker-resource
- * @short_description: Represents a single Tracker resource
- * @title: TrackerResource
- * @stability: Stable
- * @include: tracker-resource.h
+ * TrackerResource:
  *
- * #TrackerResource keeps track of a set of properties for a given resource.
- * The resulting data can be serialized in several ways.
+ * `TrackerResource` is an in-memory representation of RDF data about a given resource.
+ *
+ * This object keeps track of a set of properties for a given resource, and can
+ * also link to other `TrackerResource` objects to form trees or graphs of RDF
+ * data. See [method@Tracker.Resource.set_relation] and [method@Tracker.Resource.set_uri]
+ * on how to link a `TrackerResource` to other RDF data.
+ *
+ * `TrackerResource` may also hold data about literal values, added through
+ * the specialized [method@Tracker.Resource.set_int64], [method@Tracker.Resource.set_string],
+ * etc family of functions, or the generic [method@Tracker.Resource.set_gvalue] method.
+ *
+ * Since RDF properties may be multi-valued, for every `set` call there exists
+ * another `add` call (e.g. [method@Tracker.Resource.add_int64], [method@Tracker.Resource.add_string]
+ * and so on). The `set` methods do also reset any previously value the
+ * property might hold for the given resource.
+ *
+ * Resources may have an IRI set at creation through [ctor@Tracker.Resource.new],
+ * or set afterwards through [method@Tracker.Resource.set_identifier]. Resources
+ * without a name will represent a blank node, and will be dealt with as such
+ * during database insertions.
+ *
+ * `TrackerResource` performs no validation on the data being coherent as per
+ * any ontology. Errors will be found out at the time of using the TrackerResource
+ * for e.g. database updates.
+ *
+ * Once the RDF data is built in memory, the (tree of) `TrackerResource` may be
+ * converted to a RDF format through [method@Tracker.Resource.print_rdf], or
+ * directly inserted into a database through [method@Tracker.Batch.add_resource]
+ * or [method@Tracker.SparqlConnection.update_resource].
  */
 
 static char *
@@ -65,13 +88,6 @@ generate_blank_node_identifier (void)
 	return g_strdup_printf("_:%" G_GINT64_FORMAT, counter++);
 }
 
-/**
- * TrackerResource:
- *
- * The <structname>TrackerResource</structname> object represents information
- * about a given resource.
- */
-
 enum {
 	PROP_0,
 
@@ -247,7 +263,7 @@ set_property (GObject      *object,
  *
  * Creates a TrackerResource instance.
  *
- * Returns: a newly created #TrackerResource. Free with g_object_unref() when done
+ * Returns: a newly created `TrackerResource`.
  */
 TrackerResource *
 tracker_resource_new (const char *identifier)
@@ -269,18 +285,17 @@ tracker_resource_new (const char *identifier)
 
 /**
  * tracker_resource_set_gvalue:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to set
- * @value: an initialised #GValue
+ * @value: an initialised [struct@GObject.Value]
  *
- * State that the only value for the given property is 'value'. Any existing
- * values for 'property' will be removed.
+ * Replace any previously existing value for @property_uri with @value.
  *
  * When serialising to SPARQL, any properties that were set with this function
  * will get a corresponding DELETE statement to remove any existing values in
  * the database.
  *
- * You can pass any kind of GValue for @value, but serialization functions will
+ * You can pass any kind of [struct@GObject.Value] for @value, but serialization functions will
  * normally only be able to serialize URIs/relationships and fundamental value
  * types (string, int, etc.).
  */
@@ -382,112 +397,142 @@ value_set_uri (GValue      *value,
 
 /**
  * tracker_resource_set_boolean:
- * @self: the #TrackerResource
- * @property_uri: a string identifying the property to modify
- * @value: the property object
+ * @self: The `TrackerResource`
+ * @property_uri: A string identifying the property to modify
+ * @value: The property boolean value
  *
- * Sets a single-valued boolean object.
+ * Sets a boolean property. Replaces any previous value.
+ *
+ * This method corresponds to [xsd:boolean](xsd-ontology.html#xsd:boolean).
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_boolean, gboolean, G_TYPE_BOOLEAN, g_value_set_boolean, validate_boolean)
 
 /**
  * tracker_resource_set_double:
- * @self: the #TrackerResource
- * @property_uri: a string identifying the property to modify
- * @value: the property object
+ * @self: The `TrackerResource`
+ * @property_uri: A string identifying the property to modify
+ * @value: The property object
  *
- * Sets a single-valued double object.
+ * Sets a numeric property with double precision. Replaces any previous value.
+ *
+ * This method corresponds to [xsd:double](xsd-ontology.html#xsd:double).
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_double, double, G_TYPE_DOUBLE, g_value_set_double, validate_double)
 
 /**
  * tracker_resource_set_int:
- * @self: the #TrackerResource
- * @property_uri: a string identifying the property to modify
- * @value: the property object
+ * @self: The `TrackerResource`
+ * @property_uri: A string identifying the property to modify
+ * @value: The property object
  *
- * Sets a single-valued integer object.
+ * Sets a numeric property with integer precision. Replaces any previous value.
+ *
+ * This method corresponds to [xsd:integer](xsd-ontology.html#xsd:integer).
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_int, int, G_TYPE_INT, g_value_set_int, validate_int)
 
 /**
  * tracker_resource_set_int64:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Sets a single-valued integer object.
+ * Sets a numeric property with 64-bit integer precision. Replaces any previous value.
+ *
+ * This method corresponds to [xsd:integer](xsd-ontology.html#xsd:integer).
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_int64, gint64, G_TYPE_INT64, g_value_set_int64, validate_int64)
 
 /**
  * tracker_resource_set_relation:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @resource: the property object
  *
- * Sets a single-valued resource object as a #TrackerResource. This
- * function produces similar RDF to tracker_resource_set_uri(),
+ * Sets a resource property as a `TrackerResource`. Replaces any previous value.
+ *
+ * This method applies to properties with a [rdfs:range](rdf-ontology.html#rdfs:range)
+ * that points to a non-literal class (i.e. a subclass of
+ * [rdfs:Resource](rdf-ontology.html#rdfs:Resource)).
+ *
+ * This function produces similar RDF to [method@Tracker.Resource.set_uri],
  * although in this function the URI will depend on the identifier
  * set on @resource.
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_set_object, validate_pointer)
 
 /**
- * tracker_resource_set_take_relation: (skip)
- * @self: the #TrackerResource
+ * tracker_resource_set_take_relation:
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
- * @resource: the property object
+ * @resource: (transfer full): the property object
  *
- * Sets a single-valued resource object as a #TrackerResource. This
- * function produces similar RDF to tracker_resource_set_uri(),
+ * Sets a resource property as a `TrackerResource`. Replaces any previous value.
+ * Takes ownership on the given @resource.
+ *
+ * This method applies to properties with a [rdfs:range](rdf-ontology.html#rdfs:range)
+ * that points to a non-literal class (i.e. a subclass of
+ * [rdfs:Resource](rdf-ontology.html#rdfs:Resource)).
+ *
+ * This function produces similar RDF to [method@Tracker.Resource.set_uri],
  * although in this function the URI will depend on the identifier
- * set on @resource. This function takes ownership of @resource.
+ * set on @resource.
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_take_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_take_object, validate_pointer)
 
 /**
  * tracker_resource_set_string:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Sets a single-valued string object.
+ * Sets a string property. Replaces any previous value.
+ *
+ * This method corresponds to [xsd:string](xsd-ontology.html#xsd:string).
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_string, const char *, G_TYPE_STRING, g_value_set_string, validate_pointer)
 
 /**
  * tracker_resource_set_uri:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Sets a single-valued resource object as a string URI. This function
- * produces similar RDF to tracker_resource_set_relation(), although
+ * Sets a resource property as an URI string. Replaces any previous value.
+ *
+ * This method applies to properties with a [rdfs:range](rdf-ontology.html#rdfs:range)
+ * that points to a non-literal class (i.e. a subclass of
+ * [rdfs:Resource](rdf-ontology.html#rdfs:Resource)).
+ *
+ * This function produces similar RDF to [method@Tracker.Resource.set_relation], although
  * it requires that the URI is previously known.
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_uri, const char *, TRACKER_TYPE_URI, value_set_uri, validate_pointer)
 
 /**
  * tracker_resource_set_datetime:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Sets a single-valued GDateTime as a #TrackerResource
+ * Sets a date property as a [type@GLib.DateTime]. Replaces any previous value.
+ *
+ * This method corresponds to [xsd:date](xsd-ontology.html#xsd:date) and
+ * [xsd:dateTime](xsd-ontology.html#xsd:dateTime).
+ *
  * Since: 3.2
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_datetime, GDateTime *, G_TYPE_DATE_TIME, g_value_set_boxed, validate_pointer)
 
 /**
  * tracker_resource_add_gvalue:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to set
- * @value: an initialised #GValue
+ * @value: an initialised [struct@GObject.Value]
  *
- * Add 'value' to the list of values for given property.
+ * Add @value to the list of values for given property.
  *
- * You can pass any kind of GValue for @value, but serialization functions will
+ * You can pass any kind of [struct@GObject.Value] for @value, but serialization functions will
  * normally only be able to serialize URIs/relationships and fundamental value
  * types (string, int, etc.).
  */
@@ -595,65 +640,102 @@ tracker_resource_add_gvalue (TrackerResource *self,
 
 /**
  * tracker_resource_add_boolean:
- * @self: the #TrackerResource
- * @property_uri: a string identifying the property to modify
- * @value: the property object
+ * @self: The `TrackerResource`
+ * @property_uri: A string identifying the property to modify
+ * @value: The property boolean value
  *
- * Adds a boolean object to a multi-valued property.
+ * Adds a boolean property. Previous values for the same property are kept.
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method corresponds to [xsd:boolean](xsd-ontology.html#xsd:boolean).
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_boolean, gboolean, G_TYPE_BOOLEAN, g_value_set_boolean, validate_boolean)
 
 /**
  * tracker_resource_add_double:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Adds a double object to a multi-valued property.
+ * Adds a numeric property with double precision. Previous values for the same property are kept.
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method corresponds to [xsd:double](xsd-ontology.html#xsd:double).
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_double, double, G_TYPE_DOUBLE, g_value_set_double, validate_double)
 
 /**
  * tracker_resource_add_int:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Adds an integer object to a multi-valued property.
+ * Adds a numeric property with integer precision. Previous values for the same property are kept.
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method corresponds to [xsd:integer](xsd-ontology.html#xsd:integer).
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_int, int, G_TYPE_INT, g_value_set_int, validate_int)
 
 /**
  * tracker_resource_add_int64:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Adds an integer object to a multi-valued property.
+ * Adds a numeric property with 64-bit integer precision. Previous values for the same property are kept.
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method corresponds to [xsd:integer](xsd-ontology.html#xsd:integer).
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_int64, gint64, G_TYPE_INT64, g_value_set_int64, validate_int64)
 
 /**
  * tracker_resource_add_relation:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @resource: the property object
  *
- * Adds a resource object to a multi-valued property. This
- * function produces similar RDF to tracker_resource_add_uri(),
+ * Adds a resource property as a `TrackerResource`. Previous values for the same property are kept.
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method applies to properties with a [rdfs:range](rdf-ontology.html#rdfs:range)
+ * that points to a non-literal class (i.e. a subclass of
+ * [rdfs:Resource](rdf-ontology.html#rdfs:Resource)).
+ *
+ * This method produces similar RDF to [method@Tracker.Resource.add_uri],
  * although in this function the URI will depend on the identifier
  * set on @resource.
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_set_object, validate_pointer)
 
 /**
- * tracker_resource_add_take_relation: (skip)
- * @self: the #TrackerResource
+ * tracker_resource_add_take_relation:
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
- * @resource: the property object
+ * @resource: (transfer full): the property object
  *
- * Adds a resource object to a multi-valued property. This
- * function produces similar RDF to tracker_resource_add_uri(),
+ * Adds a resource property as a `TrackerResource`. Previous values for the same property are kept.
+ * Takes ownership on the given @resource.
+ *
+ * This method is meant to RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method applies to properties with a [rdfs:range](rdf-ontology.html#rdfs:range)
+ * that points to a non-literal class (i.e. a subclass of
+ * [rdfs:Resource](rdf-ontology.html#rdfs:Resource)).
+ *
+ * This function produces similar RDF to [method@Tracker.Resource.add_uri],
  * although in this function the URI will depend on the identifier
  * set on @resource. This function takes ownership of @resource.
  */
@@ -662,46 +744,67 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_take_relation, TrackerResource *, T
 
 /**
  * tracker_resource_add_string:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Adds a string object to a multi-valued property.
+ * Adds a string property. Previous values for the same property are kept.
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method corresponds to [xsd:string](xsd-ontology.html#xsd:string).
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_string, const char *, G_TYPE_STRING, g_value_set_string, validate_pointer)
 
 /**
  * tracker_resource_add_uri:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Adds a resource object to a multi-valued property. This function
- * produces similar RDF to tracker_resource_add_relation(), although
+ * Adds a resource property as an URI string. Previous values for the same property are kept.
+ *
+ * This method applies to properties with a [rdfs:range](rdf-ontology.html#rdfs:range)
+ * that points to a non-literal class (i.e. a subclass of
+ * [rdfs:Resource](rdf-ontology.html#rdfs:Resource)).
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This function produces similar RDF to [method@Tracker.Resource.add_relation], although
  * it requires that the URI is previously known.
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_uri, const char *, TRACKER_TYPE_URI, value_set_uri, validate_pointer)
 
 /**
  * tracker_resource_add_datetime:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to modify
  * @value: the property object
  *
- * Adds GDateTime object to the multi-valued property.
+ * Adds a date property as a [type@GLib.DateTime]. Previous values for the
+ * same property are kept.
+ *
+ * This method is meant for RDF properties allowing multiple values, see
+ * [nrl:maxCardinality](nrl-ontology.html#nrl:maxCardinality).
+ *
+ * This method corresponds to [xsd:date](xsd-ontology.html#xsd:date) and
+ * [xsd:dateTime](xsd-ontology.html#xsd:dateTime).
+ *
  * Since: 3.2
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_datetime, GDateTime *, G_TYPE_DATE_TIME, g_value_set_boxed, validate_pointer)
 
 /**
  * tracker_resource_get_values:
- * @self: the #TrackerResource
+ * @self: the `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the list of all known values of the given property.
  *
- * Returns: (transfer container) (element-type GValue): a #GList of #GValue
- * instances. The list should be freed with g_list_free()
+ * Returns: (transfer container) (element-type GValue) (nullable): a [struct@GLib.List] of
+ *   [struct@GObject.Value] instances. The list should be freed with [func@GLib.List.free]
  */
 GList *tracker_resource_get_values (TrackerResource *self,
                                     const char      *property_uri)
@@ -771,7 +874,7 @@ GList *tracker_resource_get_values (TrackerResource *self,
 
 /**
  * tracker_resource_get_first_boolean:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the first boolean object previously assigned to a property.
@@ -782,7 +885,7 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_boolean, gboolean, G_TYPE_BOO
 
 /**
  * tracker_resource_get_first_double:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the first double object previously assigned to a property.
@@ -793,7 +896,7 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_double, double, G_TYPE_DOUBLE
 
 /**
  * tracker_resource_get_first_int:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the first integer object previously assigned to a property.
@@ -804,7 +907,7 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_int, int, G_TYPE_INT, g_value
 
 /**
  * tracker_resource_get_first_int64:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the first integer object previously assigned to a property.
@@ -815,59 +918,59 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_int64, gint64, G_TYPE_INT64,
 
 /**
  * tracker_resource_get_first_relation:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the first resource object previously assigned to a property.
  *
- * Returns: (transfer none): the first resource object
+ * Returns: (transfer none) (nullable): the first resource object
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_get_object, NULL)
 
 /**
  * tracker_resource_get_first_string:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the first string object previously assigned to a property.
  *
- * Returns: the first string object
+ * Returns: (nullable): the first string object
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_string, const char *, G_TYPE_STRING, g_value_get_string, NULL)
 
 /**
  * tracker_resource_get_first_uri:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
  * Returns the first resource object previously assigned to a property.
  *
- * Returns: the first resource object as an URI.
+ * Returns: (nullable): the first resource object as an URI.
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_uri, const char *, TRACKER_TYPE_URI, g_value_get_string, NULL)
 
 /**
  * tracker_resource_get_first_datetime:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  * @property_uri: a string identifying the property to look up
  *
- * Returns the first resource object previously assigned to a property.
+ * Returns the first [type@GLib.DateTime] previously assigned to a property.
  *
- * Returns: the first GDateTime object
+ * Returns: (transfer none) (nullable): the first GDateTime object
  * Since: 3.2
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_datetime, GDateTime *, G_TYPE_DATE_TIME, g_value_get_boxed, NULL)
 
 /**
  * tracker_resource_get_identifier:
- * @self: A #TrackerResource
+ * @self: A `TrackerResource`
  *
  * Returns the identifier of a resource.
  *
- * If the identifier was set to NULL, the identifier returned will be a unique
- * SPARQL blank node identifier, such as "_:123".
+ * If the identifier was set to NULL, the identifier returned will be a locally
+ * unique SPARQL blank node identifier, such as `_:123`.
  *
- * Returns: a string owned by the resource
+ * Returns: (nullable): a string owned by the resource
  */
 const char *
 tracker_resource_get_identifier (TrackerResource *self)
@@ -886,16 +989,16 @@ tracker_resource_get_identifier (TrackerResource *self)
 
 /**
  * tracker_resource_set_identifier:
- * @self: a #TrackerResource
- * @identifier: (allow-none): a string identifying the resource
+ * @self: A `TrackerResource`
+ * @identifier: (nullable): a string identifying the resource
  *
- * Changes the identifier of a #TrackerResource. The identifier should be a
+ * Changes the identifier of a `TrackerResource`. The identifier should be a
  * URI or compact URI, but this is not necessarily enforced. Invalid
  * identifiers may cause errors when serializing the resource or trying to
  * insert the results in a database.
  *
- * If the identifier is set to NULL, a SPARQL blank node identifier such as
- * "_:123" is assigned to the resource.
+ * If the identifier is set to %NULL, a SPARQL blank node identifier such as
+ * `_:123` is assigned to the resource.
  */
 void
 tracker_resource_set_identifier (TrackerResource *self,
@@ -913,10 +1016,10 @@ tracker_resource_set_identifier (TrackerResource *self,
 
 /**
  * tracker_resource_identifier_compare_func:
- * @resource: a #TrackerResource
+ * @resource: a `TrackerResource`
  * @identifier: a string identifying the resource
  *
- * A helper function that compares a #TrackerResource by its identifier
+ * A helper function that compares a `TrackerResource` by its identifier
  * string.
  *
  * Returns: an integer less than, equal to, or greater than zero, if the
@@ -934,7 +1037,8 @@ tracker_resource_identifier_compare_func (TrackerResource *resource,
 
 /**
  * tracker_resource_compare:
- * @self: A #TrackerResource
+ * @a: A `TrackerResource`
+ * @b: A second `TrackerResource` to compare
  *
  * Compare the identifiers of two TrackerResource instances. The resources
  * are considered identical if they have the same identifier.
@@ -958,14 +1062,11 @@ tracker_resource_compare (TrackerResource *a,
 
 /**
  * tracker_resource_get_properties:
- * @resource: a #TrackerResource
+ * @resource: a `TrackerResource`
  *
  * Gets the list of properties defined in @resource
  *
  * Returns: (transfer container) (element-type utf8): The list of properties.
- *          The list should be freed with g_list_free().
- *
- * Since: 3.0
  **/
 GList *
 tracker_resource_get_properties (TrackerResource *resource)
@@ -989,8 +1090,10 @@ parse_prefix (const gchar *prefixed_name)
 	if (!terminal_PNAME_NS (prefixed_name, end, &token_end))
 		return NULL;
 
+	g_assert (token_end != NULL);
+
 	/* We have read the ':', take a step back */
-	if (token_end && token_end > prefixed_name)
+	if (token_end > prefixed_name)
 		token_end--;
 
 	if (*token_end != ':')
@@ -1190,7 +1293,7 @@ generate_turtle_property (const char              *property,
 
 /**
  * tracker_resource_print_turtle:
- * @self: a #TrackerResource
+ * @self: a `TrackerResource`
  * @namespaces: (allow-none): a set of prefixed URLs, or %NULL to use the
  *     Nepomuk set
  *
@@ -1200,12 +1303,12 @@ generate_turtle_property (const char              *property,
  * <https://www.w3.org/TR/2014/REC-turtle-20140225/>
  *
  * The @namespaces object is used to expand any compact URI values. In most
- * cases you should pass the one returned by tracker_sparql_connection_get_namespace_manager()
+ * cases you should pass the one returned by [method@Tracker.SparqlConnection.get_namespace_manager]
  * from the connection that is the intended recipient of this data.
  *
  * Returns: a newly-allocated string
  *
- * Deprecated: 3.4: Use tracker_resource_print_rdf() instead.
+ * Deprecated: 3.4: Use [method@Tracker.Resource.print_rdf] instead.
  */
 char *
 tracker_resource_print_turtle (TrackerResource         *self,
@@ -1422,7 +1525,7 @@ generate_sparql_insert_pattern (TrackerResource    *resource,
 
 /**
  * tracker_resource_print_sparql_update:
- * @self: a #TrackerResource
+ * @self: a `TrackerResource`
  * @namespaces: (allow-none): a set of prefixed URLs, or %NULL to use the
  *     Nepomuk set
  * @graph_id: (allow-none): the URN of the graph the data should be added to,
@@ -1432,7 +1535,7 @@ generate_sparql_insert_pattern (TrackerResource    *resource,
  * stored in @resource.
  *
  * The @namespaces object is used to expand any compact URI values. In most
- * cases you should pass the one returned by tracker_sparql_connection_get_namespace_manager()
+ * cases you should pass the one returned by [method@Tracker.SparqlConnection.get_namespace_manager]
  * from the connection that is the intended recipient of this data.
  *
  * Returns: a newly-allocated string containing a SPARQL update command.
@@ -1499,120 +1602,10 @@ tracker_resource_print_sparql_update (TrackerResource         *resource,
 	return g_string_free (context.string, FALSE);
 }
 
-typedef struct {
-	TrackerNamespaceManager *all_namespaces, *our_namespaces;
-	JsonBuilder *builder;
-	GList *done_list;
-} GenerateJsonldData;
-
-static void generate_jsonld_foreach (gpointer key, gpointer value_ptr, gpointer user_data);
-
-static void
-tracker_resource_generate_jsonld (TrackerResource    *self,
-                                  GenerateJsonldData *data)
-{
-	TrackerResourcePrivate *priv = GET_PRIVATE (self);
-	JsonBuilder *builder = data->builder;
-
-	/* The JSON-LD spec says it is "important that nodes have an identifier", but
-	 * doesn't mandate one. I think it's better to omit the ID for blank nodes
-	 * (where the caller passed NULL as an identifier) than to emit something
-	 * SPARQL-specific like '_:123'.
-	 */
-	if (!tracker_resource_is_blank_node (self)) {
-		json_builder_set_member_name (builder, "@id");
-		json_builder_add_string_value (builder, priv->identifier);
-	}
-
-	g_hash_table_foreach (priv->properties, generate_jsonld_foreach, data);
-}
-
-static void
-generate_jsonld_value (const GValue       *value,
-                       GenerateJsonldData *data)
-{
-	JsonNode *node;
-
-	if (G_VALUE_HOLDS (value, TRACKER_TYPE_RESOURCE)) {
-		TrackerResource *resource;
-
-		resource = TRACKER_RESOURCE (g_value_get_object (value));
-
-		if (g_list_find (data->done_list, resource) == NULL) {
-			data->done_list = g_list_prepend (data->done_list, resource);
-			json_builder_begin_object (data->builder);
-
-			tracker_resource_generate_jsonld (resource, data);
-
-			json_builder_end_object (data->builder);
-		} else {
-			json_builder_add_string_value (data->builder, tracker_resource_get_identifier(resource));
-		}
-	} else if (G_VALUE_HOLDS (value, TRACKER_TYPE_URI)) {
-		/* URIs can be treated the same as strings in JSON-LD provided the @context
-		 * sets the type of that property correctly. However, json_node_set_value()
-		 * will reject a GValue holding TRACKER_TYPE_URI, so we have to extract the
-		 * string manually here.
-		 */
-		const char *uri = g_value_get_string (value);
-		maybe_intern_prefix_of_compact_uri (data->all_namespaces, data->our_namespaces, uri);
-		node = json_node_new (JSON_NODE_VALUE);
-		json_node_set_string (node, uri);
-		json_builder_add_value (data->builder, node);
-	} else if (G_VALUE_HOLDS (value, G_TYPE_UINT)) {
-		node = json_node_new (JSON_NODE_VALUE);
-		json_node_set_int (node, g_value_get_uint(value));
-		json_builder_add_value (data->builder, node);
-	} else {
-		node = json_node_new (JSON_NODE_VALUE);
-		json_node_set_value (node, value);
-		json_builder_add_value (data->builder, node);
-	}
-}
-
-static void
-generate_jsonld_foreach (gpointer key,
-                         gpointer value_ptr,
-                         gpointer user_data)
-{
-	const char *property = key;
-	const GValue *value = value_ptr;
-	GenerateJsonldData *data = user_data;
-	JsonBuilder *builder = data->builder;
-
-	if (strcmp (property, "rdf:type") == 0) {
-		property = "@type";
-	} else {
-		maybe_intern_prefix_of_compact_uri (data->all_namespaces, data->our_namespaces, property);
-	}
-
-	json_builder_set_member_name (builder, property);
-
-	if (G_VALUE_HOLDS (value, G_TYPE_PTR_ARRAY)) {
-		json_builder_begin_array (builder);
-		g_ptr_array_foreach (g_value_get_boxed (value), (GFunc) generate_jsonld_value, data);
-		json_builder_end_array (builder);
-	} else {
-		generate_jsonld_value (value, data);
-	}
-}
-
-static void
-generate_jsonld_namespace_mapping_foreach (gpointer key,
-                                           gpointer value,
-                                           gpointer user_data)
-{
-	GenerateJsonldData *data = user_data;
-
-	json_builder_set_member_name (data->builder, key);
-	json_builder_add_string_value (data->builder, value);
-}
-
-
 /**
  * tracker_resource_print_jsonld:
- * @self: a #TrackerResource
- * @namespaces: (allow-none): a set of prefixed URLs, or %NULL to use the
+ * @self: a `TrackerResource`
+ * @namespaces: (nullable): a set of prefixed URLs, or %NULL to use the
  *     Nepomuk set
  *
  * Serialize all the information in @resource as a JSON-LD document.
@@ -1621,19 +1614,18 @@ generate_jsonld_namespace_mapping_foreach (gpointer key,
  * serialization format.
  *
  * The @namespaces object is used to expand any compact URI values. In most
- * cases you should pass the one returned by tracker_sparql_connection_get_namespace_manager()
+ * cases you should pass the one returned by [method@Tracker.SparqlConnection.get_namespace_manager]
  * from the connection that is the intended recipient of this data.
  *
  * Returns: a newly-allocated string containing JSON-LD data.
+ *
+ * Deprecated: 3.5: Use [method@Tracker.Resource.print_rdf] instead.
  */
 char *
 tracker_resource_print_jsonld (TrackerResource         *self,
                                TrackerNamespaceManager *namespaces)
 {
-	GenerateJsonldData context;
-	JsonNode *json_root_node;
-	JsonGenerator *generator;
-	char *result;
+	g_return_val_if_fail (TRACKER_IS_RESOURCE (self), "");
 
 	if (namespaces == NULL) {
 		G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -1641,38 +1633,7 @@ tracker_resource_print_jsonld (TrackerResource         *self,
 		G_GNUC_END_IGNORE_DEPRECATIONS
 	}
 
-	context.all_namespaces = namespaces;
-	context.our_namespaces = tracker_namespace_manager_new ();
-	context.builder = json_builder_new ();
-	context.done_list = g_list_prepend (NULL, self);
-
-	maybe_intern_prefix_of_compact_uri (context.all_namespaces, context.our_namespaces, tracker_resource_get_identifier (self));
-
-	json_builder_begin_object (context.builder);
-
-	tracker_resource_generate_jsonld (self, &context);
-
-	json_builder_set_member_name (context.builder, "@context");
-	json_builder_begin_object (context.builder);
-	tracker_namespace_manager_foreach (context.our_namespaces, generate_jsonld_namespace_mapping_foreach, &context);
-	json_builder_end_object (context.builder);
-
-	json_builder_end_object (context.builder);
-
-	json_root_node = json_builder_get_root (context.builder);
-
-	generator = json_generator_new ();
-	json_generator_set_root (generator, json_root_node);
-	json_generator_set_pretty (generator, TRUE);
-
-	result = json_generator_to_data (generator, NULL);
-
-	g_list_free (context.done_list);
-	json_node_free (json_root_node);
-	g_object_unref (context.builder);
-	g_object_unref (generator);
-
-	return result;
+	return tracker_resource_print_rdf (self, namespaces, TRACKER_RDF_FORMAT_JSON_LD, NULL);
 }
 
 static TrackerSerializerFormat
@@ -1683,7 +1644,9 @@ convert_format (TrackerRdfFormat format)
 		return TRACKER_SERIALIZER_FORMAT_TTL;
 	case TRACKER_RDF_FORMAT_TRIG:
 		return TRACKER_SERIALIZER_FORMAT_TRIG;
-	case TRACKER_N_RDF_FORMATS:
+	case TRACKER_RDF_FORMAT_JSON_LD:
+		return TRACKER_SERIALIZER_FORMAT_JSON_LD;
+	case TRACKER_RDF_FORMAT_LAST:
 		g_assert_not_reached ();
 	}
 
@@ -1692,15 +1655,16 @@ convert_format (TrackerRdfFormat format)
 
 /**
  * tracker_resource_print_rdf:
- * @self: a #TrackerResource
+ * @self: a `TrackerResource`
  * @namespaces: a set of prefixed URLs
  * @format: RDF format of the printed string
- * @graph: target graph of the resource RDF, or %NULL for the default graph
+ * @graph: (nullable): target graph of the resource RDF, or %NULL for the
+ * default graph
  *
  * Serialize all the information in @resource into the selected RDF format.
  *
  * The @namespaces object is used to expand any compact URI values. In most
- * cases you should pass the one returned by tracker_sparql_connection_get_namespace_manager()
+ * cases you should pass the one returned by [method@Tracker.SparqlConnection.get_namespace_manager]
  * from the connection that is the intended recipient of this data.
  *
  * Returns: a newly-allocated string containing RDF data in the requested format.
@@ -1721,7 +1685,6 @@ tracker_resource_print_rdf (TrackerResource         *self,
 	g_return_val_if_fail (TRACKER_IS_NAMESPACE_MANAGER (namespaces), NULL);
 	g_return_val_if_fail (format < TRACKER_N_RDF_FORMATS, NULL);
 
-#define BUF_SIZE 4096
 	deserializer = tracker_deserializer_resource_new (self, namespaces, graph);
 	serializer = tracker_serializer_new (TRACKER_SPARQL_CURSOR (deserializer),
 	                                     namespaces,
@@ -1730,6 +1693,34 @@ tracker_resource_print_rdf (TrackerResource         *self,
 
 	str = g_string_new (NULL);
 
+	if (format == TRACKER_RDF_FORMAT_JSON_LD) {
+		JsonParser *parser;
+		JsonGenerator *generator;
+		JsonNode *root;
+
+		/* Special case, ensure that json is pretty printed */
+		parser = json_parser_new ();
+
+		if (!json_parser_load_from_stream (parser,
+		                                   serializer,
+		                                   NULL,
+		                                   NULL)) {
+			g_object_unref (parser);
+			return g_string_free (str, FALSE);
+		}
+
+		generator = json_generator_new ();
+		root = json_parser_get_root (parser);
+		json_generator_set_root (generator, root);
+		json_generator_set_pretty (generator, TRUE);
+		json_generator_to_gstring (generator, str);
+		g_object_unref (generator);
+		g_object_unref (parser);
+
+		return g_string_free (str, FALSE);
+	}
+
+#define BUF_SIZE 4096
 	while (TRUE) {
 		GBytes *bytes;
 
@@ -1786,13 +1777,13 @@ tracker_serialize_single_value (TrackerResource         *resource,
 
 /**
  * tracker_resource_serialize:
- * @resource: A #TrackerResource
+ * @resource: A `TrackerResource`
  *
- * Serializes a #TrackerResource to a #GVariant in a lossless way.
+ * Serializes a `TrackerResource` to a [type@GLib.Variant] in a lossless way.
  * All child resources are subsequently serialized. It is implied
- * that both ends use a common #TrackerNamespaceManager.
+ * that both ends use a common [class@Tracker.NamespaceManager].
  *
- * Returns: (transfer full): A variant describing the resource,
+ * Returns: (transfer floating) (nullable): A variant describing the resource,
  *          the reference is floating.
  **/
 GVariant *
@@ -1862,13 +1853,13 @@ tracker_resource_serialize (TrackerResource *resource)
 
 /**
  * tracker_resource_deserialize:
- * @variant: a #GVariant
+ * @variant: a [type@GLib.Variant]
  *
- * Deserializes a #TrackerResource previously serialized with
- * tracker_resource_serialize(). It is implied that both ends
- * use a common #TrackerNamespaceManager.
+ * Deserializes a `TrackerResource` previously serialized with
+ * [method@Tracker.Resource.serialize]. It is implied that both ends
+ * use a common [class@Tracker.NamespaceManager].
  *
- * Returns: (transfer full): A TrackerResource, or %NULL if
+ * Returns: (transfer full) (nullable): A TrackerResource, or %NULL if
  *          deserialization fails.
  **/
 TrackerResource *
@@ -1983,7 +1974,7 @@ tracker_resource_deserialize (GVariant *variant)
 
 /**
  * tracker_resource_get_property_overwrite:
- * @resource: a #TrackerResource
+ * @resource: a `TrackerResource`
  * @property_uri: a string identifying the property to query
  *
  * Returns whether the prior values for this property would be deleted
